/******************************************************************************
 * effectpop.cpp
 * Sound effect remote control dialog.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>
#include <math.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                     DIALOG CALLBACK                      ----!!!!
 ******************************************************************************/
#define PRIORITY_SLIDER_MIN	1
#define PRIORITY_SLIDER_MAX	127

#define PANNING_SLIDER_MIN	0
#define PANNING_SLIDER_CEN	128
#define PANNING_SLIDER_MAX	256

/* BOOL CALLBACK musicProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 *  Callback for the "Sound Effects Remote" dialog.
 *----------------------------------------------------------------------------*/
BOOL CALLBACK sfxProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		/**
		 * ---- WM_INITDIALOG ----
		 * Received once the dialog is ready and all the controls have
		 * been created, used to set up anything akin to WM_CREATE's
		 * traditional use in normal windows.
		 */
		case WM_INITDIALOG:
			break;

		/**
		 * ---- WM_CLOSE ----
		 */
		case WM_CLOSE:
			EndDialog(hWnd, IDCANCEL);
			pEffectPop->closePop(false);
			break;



		/**
		 * ---- WM_HSCROLL ----
		 * Somewhere, a trackbar has moved.
		 */
		case WM_HSCROLL:
			pEffectPop->refresh();
			break;

		/**
		 * ---- WM_COMMAND ----
		 * Used for button presses in this particular dialog.
		 */
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDC_SFX_OPEN:
					SendMessage(GetParent(hWnd),
						WM_COMMAND, IDM_FILE_OPEN_SFX, 0);
					break;

				case IDC_SFX_PLAY:
					if(pBupMan->mbGotSFX)
					{
						pEffectPop->dispatch();
					}
					break;

				case IDC_SFX_STOP:
					pBupMan->stopSFX();
					break;
			}

			pEffectPop->refresh();
			break;

		/**
		 * Who knows?
		 * However, we -do- know not to use DefWindowProc here,
		 * it's absolutely not for dialogs.
		 */
		default:
			return FALSE;
	}

	return TRUE;
}




/******************************************************************************
 * !!!!----                DYNAMIC CONTENT CONTROL                   ----!!!!
 ******************************************************************************/
/* void EffectPop::refresh(void)
 *----------------------------------------------------------------------------*/
void EffectPop::refresh(void)
{
	INT iCon;
	HICON hCon;

	if(NULL != mhPop)
	{
		/* ---- SLIDERS ----
		 */
		readSliders();
		snapSliders();

		/* ---- STATUS BOX ----
		 */
		pStatMan->clipFile(mszSFX, pMenuMan->mszSFX, (TCHAR*)szTerms);
		sprintf(mszRender, _T("Sound : %s\n\nPriority : %d\nPanning : %u,%u"),
			mszSFX,
			miPri,
			miLeft, miRight);
		SetWindowText(mhStatus, mszRender);


		/* ---- BUTTONS ----
		 */
		iCon = pBupMan->mbGotSFX
			? IDI_OPEN_DONE : IDI_OPEN_READY;
		hCon = pPanelMan->getIcon(iCon);
		SendMessage(mhOpen, BM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)hCon);

		iCon = pBupMan->mbGotSFX
			? IDI_PLAY_READY : IDI_PLAY_GREY;
		hCon = pPanelMan->getIcon(iCon);
		SendMessage(mhPlay, BM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)hCon);

		iCon = pBupMan->mbGotSFX
			? IDI_STOP_READY : IDI_STOP_GREY;
		hCon = pPanelMan->getIcon(iCon);
		SendMessage(mhStop, BM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)hCon);
	}
}

/* void EffectPop::dispatch(void)
 *----------------------------------------------------------------------------*/
void EffectPop::dispatch(void)
{
	pBupMan->playSFX(miPri, miLeft, miRight);
}

/* void EffectPop::readSliders(void)
 *----------------------------------------------------------------------------*/
void EffectPop::readSliders(void)
{
	UINT uiPri,uiPan,uiPrev;

	if(NULL != mhPri)
	{
		uiPri = SendMessage(mhPri, TBM_GETPOS, 0, 0);
		uiPrev = miPri;
		if(uiPrev != uiPri)
		{
			pBupMan->stopSFX();
		}

		miPri = uiPri;
	}

	if(NULL != mhPan)
	{
		uiPan = SendMessage(mhPan, TBM_GETPOS, 0, 0);
		miCenter = uiPan;

		miLeft = PANNING_SLIDER_CEN - (miCenter - PANNING_SLIDER_CEN);
		if(miLeft < 0)
		{
			miLeft = 0;
		}
		else if(miLeft > 127)
		{
			miLeft = 127;
		}

		miRight = PANNING_SLIDER_CEN - (PANNING_SLIDER_CEN - miCenter);
		if(miRight < 0)
		{
			miRight = 0;
		}
		else if(miRight > 127)
		{
			miRight = 127;
		}
	}
}

/* void EffectPop::snapSliders(void)
 *----------------------------------------------------------------------------*/
void EffectPop::snapSliders(void)
{
	UINT uiPri,uiPan;

	if(NULL != mhPri)
	{
		uiPri = miPri;
		SendMessage(mhPri, TBM_SETPOS, (WPARAM)TRUE, (LPARAM)uiPri);
	}

	if(NULL != mhPan)
	{
		uiPan = miCenter;
		SendMessage(mhPan, TBM_SETPOS, (WPARAM)TRUE, (LPARAM)uiPan);
	}
}




/******************************************************************************
 * !!!!----                POPUP DIALOG MANAGEMENT                   ----!!!!
 ******************************************************************************/
/* BOOL EffectPop::createPop(void)
 *----------------------------------------------------------------------------*/
BOOL EffectPop::createPop(void)
{
	if(!IsWindow(mhPop))
	{
		mhPop = CreateDialog(GetModuleHandle(NULL),
			MAKEINTRESOURCE(IDD_SFX), mhParent, (DLGPROC)sfxProc); 

		mhStatus = GetDlgItem(mhPop, IDC_SFX_STATUS);
		mhOpen = GetDlgItem(mhPop, IDC_SFX_OPEN);
		mhPlay = GetDlgItem(mhPop, IDC_SFX_PLAY);
		mhStop = GetDlgItem(mhPop, IDC_SFX_STOP);

		mhPri = GetDlgItem(mhPop, IDC_SFX_PRI);
		mhPan = GetDlgItem(mhPop, IDC_SFX_PAN);

		SendMessage(mhPri, TBM_SETRANGE,
				(WPARAM)TRUE,
				(LPARAM)MAKELONG(PRIORITY_SLIDER_MIN, PRIORITY_SLIDER_MAX));
		SendMessage(mhPan, TBM_SETRANGE,
			(WPARAM)TRUE,
			(LPARAM)MAKELONG(PANNING_SLIDER_MIN, PANNING_SLIDER_MAX));
		snapSliders();

		refresh();
		ShowWindow(mhPop, SW_SHOW);
	}

	return IsWindow(mhPop);
}

/* void EffectPop::closePop(BOOL bForce)
 *----------------------------------------------------------------------------*/
void EffectPop::closePop(BOOL bForce)
{
	if(bForce && IsWindow(mhPop))
	{
		SendMessage(mhPop, WM_CLOSE, 0, 0);
	}

	mhPri = NULL;
	mhPan = NULL;

	mhStatus = NULL;
	mhOpen = NULL;
	mhPlay = NULL;
	mhStop = NULL;

	mhPop = NULL;
}

/* BOOL EffectPop::checkPop(void)
 *----------------------------------------------------------------------------*/
BOOL EffectPop::checkPop(void)
{
	return IsWindow(mhPop);
}

/* HWND EffectPop::getPop(void)
 *----------------------------------------------------------------------------*/
HWND EffectPop::getPop(void)
{
	return mhPop;
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
EffectPop::EffectPop(HWND hParent)
{
	mhParent = hParent;

	mhPop = NULL;
	mhPri = NULL;
	mhPan = NULL;

	miPri = 64;
	miCenter = 128;
	miLeft = 127;
	miRight = 127;

	mhStatus = NULL;
	mhOpen = NULL;
	mhPlay = NULL;
	mhStop = NULL;
}

EffectPop::~EffectPop(void)
{
	
}
