/******************************************************************************
 * musicpop.cpp
 * Music activity dialog.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>
#include <math.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                     DIALOG CALLBACK                      ----!!!!
 ******************************************************************************/
/* BOOL CALLBACK musicProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 *  Callback for the "Music Activity" dialog.
 *----------------------------------------------------------------------------*/
BOOL CALLBACK musicProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		/**
		 * ---- WM_INITDIALOG ----
		 * Received once the dialog is ready and all the controls have
		 * been created, used to set up anything akin to WM_CREATE's
		 * traditional use in normal windows.
		 */
		case WM_INITDIALOG:
			pMusicPop->setupList(hWnd);
			break;

		/**
		 * ---- WM_CLOSE ----
		 */
		case WM_CLOSE:
			EndDialog(hWnd, IDCANCEL);
			pMusicPop->closePop(false);
			break;

		/**
		 * Who knows?
		 * However, we -do- know not to use DefWindowProc here,
		 * it's absolutely not for dialogs.
		 */
		default:
			return FALSE;
	}

	return TRUE;
}




/******************************************************************************
 * !!!!----                  LIST CONTENT CONTROL                    ----!!!!
 ******************************************************************************/
const struct {
	TCHAR *szName;
	INT fmt,width;
} aMusicLabels[] = {
	{_T("Channel"),			LVCFMT_LEFT,			52},
	{_T("Priority"),		LVCFMT_CENTER,			48},
	{_T("Macro"),			LVCFMT_CENTER,			48},
	{_T("Note"),			LVCFMT_CENTER,			40},
	{_T("Pan (L)"),			LVCFMT_CENTER,			48},
	{_T("Pan (R)"),			LVCFMT_CENTER,			48},
	{_T("Decode Offset"),	LVCFMT_CENTER,			84},
	{_T("Decode Delay"),	LVCFMT_CENTER,			80},
	{_T("Stack Level"),		LVCFMT_CENTER,			72}
};

#define MUSICPOP_COLUMN_CHANNEL			0
#define MUSICPOP_COLUMN_PRIORITY		1
#define MUSICPOP_COLUMN_MACRO			2
#define MUSICPOP_COLUMN_NOTE			3
#define MUSICPOP_COLUMN_DECODE_OFF		4
#define MUSICPOP_COLUMN_DECODE_DEL		5
#define MUSICPOP_COLUMN_STACK			6
#define MUSICPOP_COLUMN_PAN_L			7
#define MUSICPOP_COLUMN_PAN_R			8
#define MUSICPOP_COLUMNS				9

#define MUSICPOP_ITEM_SIZE				32

const TCHAR *aszNotes[] = {
	_T("c."),_T("cs"),_T("d."),_T("ds"),_T("e."),_T("f."),
	_T("fs"),_T("g."),_T("gs"),_T("a."),_T("as"),_T("b."),
	NULL
};


/* void MusicPop::setupList(HWND hWnd)
 *----------------------------------------------------------------------------*/
void MusicPop::setupList(HWND hWnd)
{
	LVCOLUMN lvCol;
	LVITEM lvItem;

	UINT uiC,uiR;
	TCHAR *pszCur;

	if(NULL == mhList)
	{
		mhList = GetDlgItem(hWnd, IDC_MUSIC_LIST);

		/* ---- COLUMNS ----
		 */
		ZeroMemory(&lvCol, sizeof(LVCOLUMN));
		lvCol.mask = (LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM);
		lvCol.fmt = aMusicLabels[0].fmt;
		lvCol.cx = aMusicLabels[0].width;
		lvCol.pszText = (TCHAR*)(aMusicLabels[0].szName);
		lvCol.cchTextMax = (lstrlen(aMusicLabels[0].szName) + 1);
		lvCol.iSubItem = 0;

		SendMessage(mhList, LVM_INSERTCOLUMN, 0, (LPARAM)&lvCol);

		for(uiC = 1; uiC < MUSICPOP_COLUMNS; uiC++)
		{
			lvCol.fmt = aMusicLabels[uiC].fmt;
			lvCol.cx = aMusicLabels[uiC].width;
			lvCol.pszText = (TCHAR*)(aMusicLabels[uiC].szName);
			lvCol.cchTextMax = (lstrlen(aMusicLabels[uiC].szName) + 1);
			lvCol.iSubItem = uiC;

			SendMessage(mhList, LVM_INSERTCOLUMN, uiC, (LPARAM)&lvCol);
		}


		/* ---- CONTENTS ----
		 */
		pollTone();

		ZeroMemory(&lvItem, sizeof(LVITEM));
		lvItem.mask = LVIF_TEXT;
		lvItem.cchTextMax = MUSICPOP_ITEM_SIZE;

		pszCur = mpszItems;
		for(uiR = 0; uiR < muiRows; uiR++)
		{
			lvItem.pszText = pszCur;
			pszCur += MUSICPOP_ITEM_SIZE;

			lvItem.iItem = uiR;
			lvItem.iSubItem = 0;
			SendMessage(mhList, LVM_INSERTITEM, 0, (LPARAM)&lvItem);

			for(uiC = 1; uiC < MUSICPOP_COLUMNS; uiC++)
			{
				lvItem.pszText = pszCur;
				pszCur += MUSICPOP_ITEM_SIZE;

				lvItem.iSubItem = uiC;
				SendMessage(mhList, LVM_SETITEM, 0, (LPARAM)&lvItem);
			}
		}
	}
}

/* void MusicPop::pushList(void)
 *----------------------------------------------------------------------------*/
void MusicPop::pushList(void)
{
	LVITEM lvItem;

	UINT uiC,uiR;
	TCHAR *pszCur;

	if(NULL != mhList)
	{
		pollTone();

		ZeroMemory(&lvItem, sizeof(LVITEM));
		lvItem.mask = LVIF_TEXT;
		lvItem.cchTextMax = MUSICPOP_ITEM_SIZE;

		pszCur = mpszItems;
		for(uiR = 0; uiR < muiRows; uiR++)
		{
			pszCur += MUSICPOP_ITEM_SIZE;

			lvItem.iItem = uiR;
			for(uiC = 1; uiC < MUSICPOP_COLUMNS; uiC++)
			{
				lvItem.pszText = pszCur;
				pszCur += MUSICPOP_ITEM_SIZE;

				lvItem.iSubItem = uiC;
				SendMessage(mhList, LVM_SETITEM, 0, (LPARAM)&lvItem);
			}
		}
	}
}

/* void MusicPop::pollTone(void)
 *----------------------------------------------------------------------------*/
void MusicPop::pollTone(void)
{
	CoreTrack_t *pTracks;

	UINT uiCur,uiNote;
	TCHAR *pszCur;

	wt_getState(NULL, NULL, (VOID**)(&pTracks), NULL, NULL, NULL, NULL);
	pszCur = mpszItems;
	for(uiCur = 0; uiCur < muiRows; uiCur++)
	{
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;

		/* PRIORITY
		 */
		sprintf(pszCur, _T("%d"), pTracks[uiCur].iPriority);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;


		/* MACRO,NOTE
		 */
		sprintf(pszCur, _T("%u"), pTracks[uiCur].uiInstSel);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;

		uiNote = pTracks[uiCur].ucNote;
		if(uiNote < CORETONE_MUSIC_NOTE_INVALID)
		{
			sprintf(pszCur, _T("%s%u"), aszNotes[(uiNote % 12)], (uiNote / 12));
		}
		else
		{
			lstrcpy(pszCur, _T("--"));
		}
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;


		/* PAN (L,R)
		 */
		sprintf(pszCur, _T("%d"), pTracks[uiCur].cVolLeft);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;
		sprintf(pszCur, _T("%d"), pTracks[uiCur].cVolRight);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;


		/* DECODE OFFSET,DELAY
		 */
		sprintf(pszCur, _T("%d"), pTracks[uiCur].uiOffset);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;
		sprintf(pszCur, _T("%u"), pTracks[uiCur].uiDel);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;

		/* STACK LEVEL
		 */
		sprintf(pszCur, _T("%u"), pTracks[uiCur].uiStackPos);
		pszCur = pszCur + MUSICPOP_ITEM_SIZE;
	}
}




/******************************************************************************
 * !!!!----                POPUP DIALOG MANAGEMENT                   ----!!!!
 ******************************************************************************/
/* BOOL MusicPop::createPop(void)
 *----------------------------------------------------------------------------*/
BOOL MusicPop::createPop(void)
{
	if(!IsWindow(mhPop))
	{
		mhPop = CreateDialog(GetModuleHandle(NULL),
			MAKEINTRESOURCE(IDD_MUSIC), mhParent, (DLGPROC)musicProc); 

		ShowWindow(mhPop, SW_SHOW);
	}

	return IsWindow(mhPop);
}

/* void MusicPop::closePop(BOOL bForce)
 *----------------------------------------------------------------------------*/
void MusicPop::closePop(BOOL bForce)
{
	if(bForce && IsWindow(mhPop))
	{
		SendMessage(mhPop, WM_CLOSE, 0, 0);
	}

	mhList = NULL;
	mhPop = NULL;
}

/* BOOL MusicPop::checkPop(void)
 *----------------------------------------------------------------------------*/
BOOL MusicPop::checkPop(void)
{
	return IsWindow(mhPop);
}

/* HWND MusicPop::getPop(void)
 *----------------------------------------------------------------------------*/
HWND MusicPop::getPop(void)
{
	return mhPop;
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
MusicPop::MusicPop(HWND hParent)
{
	UINT uiR,uiC;
	TCHAR *pszCur;

	mhParent = hParent;

	mhPop = NULL;
	mhList = NULL;
	wt_getInfo(&muiRows, NULL, NULL, NULL, NULL);

	mpszItems = new TCHAR[(muiRows * MUSICPOP_COLUMNS * MUSICPOP_ITEM_SIZE)];
	pszCur = mpszItems;
	for(uiR = 0; uiR < muiRows; uiR++)
	{
		sprintf(pszCur, _T("%u"), uiR);
		pszCur += MUSICPOP_ITEM_SIZE;

		for(uiC = 1; uiC < MUSICPOP_COLUMNS; uiC++)
		{
			lstrcpy(pszCur, _T(""));;
			pszCur += MUSICPOP_ITEM_SIZE;
		}
	}
}

MusicPop::~MusicPop(void)
{
	if(NULL != mpszItems)
	{
		delete[] mpszItems;
		mpszItems = NULL;
	}
}
