/******************************************************************************
 * statman.hpp
 * General driver status and loaded files.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                      STAT LAYOUT                         ----!!!!
 ******************************************************************************/
const TCHAR szStatName[] = _T("Stat View Border");
const TCHAR szStats[] = _T(
	"[%s] %u ticks, %.2lf sec\n"
	"Samples : %s\n"
	"Macros : %s\n"
	"Music : %s, [Mood : $%x]");

const TCHAR szStats_I[] = _T("Idle");
const TCHAR szStats_R[] = _T("Ready");


/******************************************************************************
 * !!!!----                   RENDERING HANDLERS                     ----!!!!
 ******************************************************************************/
/* LRESULT StatMan::paintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
 *----------------------------------------------------------------------------*/
LRESULT StatMan::paintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
{
	double dTicks,dRate,dSec;

	UINT uiTicks,uiRate;
	INT iMood;

	wt_getState(NULL, NULL, NULL, NULL, &uiTicks, NULL, NULL);
	wt_getInfo(NULL, NULL, &uiRate, NULL, NULL);
	iMood = wt_getMood();

	dTicks = uiTicks;
	dRate = uiRate;
	dSec = dTicks / dRate;

	clipFile(mszSample, pMenuMan->mszSample, (TCHAR*)szTerms);
	clipFile(mszMacros, pMenuMan->mszMacros, (TCHAR*)szTerms);
	clipFile(mszMusic, pMenuMan->mszMusic, (TCHAR*)szTerms);

	sprintf(mszRender, szStats,
		(pBupMan->mbRunning ? szStats_R : szStats_I), uiTicks, dSec,
		mszSample, mszMacros, mszMusic,
		iMood);

	SetWindowText(mhBorder, mszRender);
	return 0;
}

/* void StatMan::clipFile(TCHAR *pszDest, TCHAR *pszSource, TCHAR *pszTerms)
 *----------------------------------------------------------------------------*/
void StatMan::clipFile(TCHAR *pszDest, TCHAR *pszSource, TCHAR *pszTerms)
{
	BOOL bDone = false;

	UINT uiLen;
	UINT uiX,uiY,uiZ;

	pszDest[0] = _T('\0');

	uiLen = lstrlen(pszSource);
	uiZ = lstrlen(pszTerms);
	if(uiLen > 1)
	{
		uiY = uiLen - 1;
		for(uiY = uiLen - 1; uiY > 0; uiY--)
		{
			for(uiX = 0; uiX < uiZ; uiX++)
			{
				if(pszSource[uiY] == pszTerms[uiX])
				{
					bDone = true;
					uiY++;

					break;
				}
			}

			if(bDone)
			{
				break;
			}
		}

		lstrcpy(pszDest, (pszSource + uiY));
	}
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
StatMan::StatMan(HWND hParent, INT iX, INT iY, INT iW, INT iH)
{
	HFONT hFont;

	mhParent = hParent;
	miX = iX;
	miY = iY;

	miWidth = iW;
	miHeight = iH;

	mhBorder = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szStatName,								/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_SUNKEN | SS_LEFT),			/* dwStyle */
		miX,									/* x */
		miY,									/* y */
		miWidth,								/* nWidth */
		miHeight,								/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)STATMAN_ID_BORDER,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder, szEmpty);

	hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	SendMessage(mhBorder, WM_SETFONT, (WPARAM)hFont, 0);
}

StatMan::~StatMan(void)
{

}
