/******************************************************************************
 * waveman.cpp
 * Dot matrix of questionable quality stereo sound.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>
#include <math.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                      WAVE LAYOUT                         ----!!!!
 ******************************************************************************/
#define WAVE_BORDER_WIDTH		2
#define WAVE_BORDER_HEIGHT		2

const TCHAR szWaveName[] = _T("Wave View Border");


/******************************************************************************
 * !!!!----                   RENDERING HANDLERS                     ----!!!!
 ******************************************************************************/
/* void WaveMan::renderWave(HDC hDCm, COLORREF crLeft, COLORREF crRight,
 *                          COLORREF crMerge)
 * Render the current stereo waveform to hDCm, which should already be
 * initialized with the dimensions miWidth by miHeight.
 *----------------------------------------------------------------------------*/
void WaveMan::renderWave(HDC hDCm, COLORREF crLeft, COLORREF crRight,
						 COLORREF crMerge)
{
	double dSrcW,dDestW;
	double dAdj,dAdj_int,dAdj_frac;
	int32p32_t lSrc,lAdj;

	int16_t sSample;
	int16_t *psWave;
	UINT uiWaveLen,uiChanLen;

	UINT uiVertAdj,uiCenter;
	UINT uiX,uiY,uiZ;

	wt_getState(NULL, NULL, NULL, NULL, NULL, &psWave, &uiWaveLen);
	uiChanLen = uiWaveLen >> 1;

	uiVertAdj = 65536 / miHeight;
	uiCenter = miHeight / 2;

	dSrcW = uiChanLen;
	dDestW = miWidth;
	dAdj = dSrcW / dDestW;

	dAdj_int = floor(dAdj);
	dAdj_frac = dAdj - dAdj_int;
	dAdj_frac *= 4294967296.0;
	lAdj.uiPair.uiHi = (uint32_t)dAdj_int;
	lAdj.uiPair.uiLo = (uint32_t)dAdj_frac;

	uiX = 0;
	uiZ = miWidth;
	lSrc.lWhole = 0;
	while((uiX < uiZ) && (lSrc.uiPair.uiHi < uiChanLen))
	{
		/**
		 * <<<<---- LEFT Channel
		 */
		sSample = -1 * psWave[(lSrc.uiPair.uiHi * 2) + 0];
		uiY = sSample;
		uiY = (uiY + 32768) / uiVertAdj;

		if(uiY >= uiCenter)
		{
			while(uiY >= uiCenter)
			{
				SetPixel(hDCm, uiX, uiY, crLeft);
				uiY--;
			}
		}
		else
		{
			while(uiY < uiCenter)
			{
				SetPixel(hDCm, uiX, uiY, crLeft);
				uiY++;
			}
		}

		/* ---->>>> RIGHT Channel
		 */
		sSample = -1 * psWave[(lSrc.uiPair.uiHi * 2) + 1];
		uiY = sSample;
		uiY = (uiY + 32768) / uiVertAdj;

		if(uiY >= uiCenter)
		{
			while(uiY >= uiCenter)
			{
				SetPixel(hDCm, uiX, uiY,
					((GetPixel(hDCm, uiX, uiY) == crLeft)
						? crMerge
						: crRight));
				uiY--;
			}
		}
		else
		{
			while(uiY < uiCenter)
			{
				SetPixel(hDCm, uiX, uiY,
					((GetPixel(hDCm, uiX, uiY) == crLeft)
						? crMerge
						: crRight));
				uiY++;
			}
		}

		lSrc.lWhole += lAdj.lWhole;
		uiX++;
	}
}

/* LRESULT WaveMan::paintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
 *----------------------------------------------------------------------------*/
LRESULT WaveMan::paintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
{
	HBRUSH hBrushFill,hBrushPrev;
	HBITMAP hPrevMap;

	COLORREF crWindow;
	COLORREF crLeft,crRight,crBoth;

	/**
	 * Set up the colors we're using to draw text and the wave view.
	 *
	 * Note that the values we'll get from PALETTERGB are not necessarily the
	 * true draw keys due to varying color depths.
	 */
	crWindow = GetSysColor(COLOR_3DFACE);
	crLeft = PALETTERGB(0xFF,0x00,0x00);
	crRight = PALETTERGB(0x00,0x00,0xFF);
	crBoth = PALETTERGB(0xFF,0x00,0xFF);

	if(NULL == mhDCm)
	{
		mhDCm = CreateCompatibleDC(hDC);
	}
	if(NULL != mhDCm)
	{
		if(NULL == mhWaveMap)
		{
			mhWaveMap = CreateCompatibleBitmap(hDC, miWidth, miHeight);
		}

		if(NULL != mhWaveMap)
		{
			/**
			 * Alright, we've got our display compatible bitmap ready to go.
			 * We're going to render our wave in four passes:
			 *
			 * 1) Force our COLORREFs to match the pixel properties of
			 *    the destination bitmap by doing a Set(),Get() on each.
			 * 2) Clear the bitmap to the window color.
			 * 3,4) Draw the LEFT & RIGHT channel waveforms.
			 */
			hPrevMap = (HBITMAP)SelectObject(mhDCm, mhWaveMap);
			crLeft = SetPixel(mhDCm, 0, 0, crLeft), GetPixel(mhDCm, 0, 0);
			crRight = SetPixel(mhDCm, 0, 0, crRight), GetPixel(mhDCm, 0, 0);
			crBoth = SetPixel(mhDCm, 0, 0, crBoth), GetPixel(mhDCm, 0, 0);

			hBrushFill = CreateSolidBrush(crWindow);
			hBrushPrev = (HBRUSH)SelectObject(mhDCm, hBrushFill);
			Rectangle(mhDCm, 0, 0, miWidth, miHeight);
			SelectObject(mhDCm, hBrushPrev);
			DeleteObject(hBrushFill);

			renderWave(mhDCm, crLeft, crRight, crBoth);

			/**
			 * Bit the Blits, er... Blit the Bits!
			 */
			BitBlt(hDC,
				miX, miY, miWidth, miHeight,
				mhDCm, 0, 0,
				SRCCOPY);

			SelectObject(mhDCm, hPrevMap);
		}
	}

	return 0;
}

/* LRESULT WaveMan::invalProc(HWND hWnd)
 *----------------------------------------------------------------------------*/
LRESULT WaveMan::invalProc(HWND hWnd)
{
	RECT rVue;

	rVue.left = miX + WAVE_BORDER_WIDTH;
	rVue.right = (miX + miWidth) - WAVE_BORDER_WIDTH;
	rVue.top = miY + WAVE_BORDER_HEIGHT;
	rVue.bottom = (miY + miHeight) - WAVE_BORDER_HEIGHT;

	InvalidateRect(hWnd, &rVue, FALSE);
	return 0;
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
WaveMan::WaveMan(HWND hParent, INT iX, INT iY, INT iW, INT iH)
{
	mhParent = hParent;
	miX = iX;
	miY = iY;

	miWidth = iW;
	miHeight = iH;

	mhDCm = NULL;
	mhWaveMap = NULL;

	mhBorder = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szWaveName,								/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		miX,									/* x */
		miY,									/* y */
		miWidth,								/* nWidth */
		miHeight,								/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)WAVEMAN_ID_BORDER,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder, szEmpty);
}

WaveMan::~WaveMan(void)
{
	if(NULL != mhWaveMap)
	{
		DeleteObject(mhWaveMap);
		mhWaveMap = NULL;
	}

	if(NULL != mhDCm)
	{
		DeleteDC(mhDCm);
		mhDCm = NULL;
	}
}
